import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { HeroSectionLeftCmsRepository } from "./hero-section-left-cms.repository";
import { UpdateHeroSectionLeftCmsDto } from "./dto/update-hero-section-left-cms.dto";
import { HeroSectionLeftCmsEntity } from "./entities/hero-section-left-cms.entity";
import { DoSpacesService } from "src/modules/SpacesModule/SpacesService/doSpacesService";

@Injectable()
export class HeroSectionLeftCmsService {
    constructor(
        @InjectRepository(HeroSectionLeftCmsRepository)
        private readonly repository: HeroSectionLeftCmsRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async updateOrCreateData(
        dto: UpdateHeroSectionLeftCmsDto
    ): Promise<HeroSectionLeftCmsEntity | Error> {
        const isExists = await this.repository.find({ isDeleted: false });
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<HeroSectionLeftCmsEntity[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}