import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { TermsOfServiceCmsRepository } from "./terms-of-service-cms.repository";
import { UpdateTermsOfServiceCmsDto } from "./dto/update-terms-of-service-cms.dto";
import { TermsOfServiceCms } from "./entities/terms-of-service-cms.entity";
import { UploadedMulterFileI } from "../../SpacesModule/SpacesService";
import { DoSpacesService } from "../../SpacesModule/SpacesService/doSpacesService";

@Injectable()
export class TermsOfServiceCmsService {
    constructor(
        @InjectRepository(TermsOfServiceCmsRepository)
        private readonly repository: TermsOfServiceCmsRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async updateOrCreateData(
        dto: UpdateTermsOfServiceCmsDto,
        files: {
            logo: UploadedMulterFileI
        }
    ): Promise<TermsOfServiceCms | Error> {
        if (files && files.logo) {
            const logo: any = await this.doSpaceService.uploadFile(files.logo[0], "TermsOfServiceCMS/");
            dto.logo = logo;
        }
        const isExists = await this.repository.find({ isDeleted: false });
        if (Object.keys(isExists).length === 0) {
            const data = await this.repository.createEntity(dto);
            if (!data) {
                throw new HttpException(
                    Constants.NOT_FOUND,
                    HttpStatus.NOT_FOUND
                );
            }
            return ResponseUtils.successResponseHandler(201, "Data created successfully!", "data", data);
        } else {
            let id = isExists[0].id;
            const data = await this.repository.updateEntity(dto, id);
            if (!data) {
                throw new HttpException(
                    Constants.UPDATE_FAILED,
                    HttpStatus.BAD_REQUEST
                );
            }
            return ResponseUtils.successResponseHandler(200, "Data updated successfully!", "data", data);
        }
    }

    async findAll(): Promise<TermsOfServiceCms[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }
}