import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateBlogTagDto } from "./dto/create-blog-tag.dto";
import { UpdateBlogTagDto } from "./dto/update-blog-tag.dto";
import { BlogTag } from "./entities/blog-tag.entity";

@EntityRepository(BlogTag)
export class BlogTagRepository extends Repository<BlogTag> {

    async createEntity(dto: CreateBlogTagDto): Promise<BlogTag | Error> {
        try {
            const { tagName } = dto;
            const entity = this.create({ tagName });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<BlogTag[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<BlogTag | Error> {
        try {
            return await this.findOne({ where: { id: id } });
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateBlogTagDto): Promise<BlogTag | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne({ where: { id: id } });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(BlogTag)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }

    async findTagByTagName(tagName: string): Promise<BlogTag[] | Error> {
        try {
            return await this.find({
                where: {
                    tagName: tagName,
                    isDeleted: false
                }
            });
        } catch (e) {
            return e;
        }
    }
}