import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateChangePasswordTokenDto } from "./dto/create-change-password-token.dto";
import { UpdateChangePasswordTokenDto } from "./dto/update-change-password-token.dto";
import { ChangePasswordToken } from "./entities/change-password-token.entity";


@EntityRepository(ChangePasswordToken)
export class ChangePasswordTokenRepository extends Repository<ChangePasswordToken> {

    async createChangePasswordToken(dto: CreateChangePasswordTokenDto): Promise<ChangePasswordToken | Error> {
        try {
            const { email, token } = dto;
            const entity = await this.create({
                email,
                token
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }


    async updateChangePasswordToken(id: string, dto: UpdateChangePasswordTokenDto): Promise<ChangePasswordToken | Error> {
        try {
            await this.update(id, dto);
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findAllChangePasswordToken(): Promise<ChangePasswordToken[]> {
        try {
            return await this.find({
                where: {
                    isDeleted: false
                }
            });
        } catch (e) {
            return [];
        }
    }

    async findByCredential(credential: string): Promise<ChangePasswordToken | Error> {
        try {
            // return await this.findOne(credential);
            return await this.findOne({
                where: [
                    {
                        name: credential
                    }
                ]
            });
        } catch (e) {
            return null;
        }
    }

    async findByHashToken(credential: string): Promise<ChangePasswordToken | Error> {
        try {
            // return await this.findOne(credential);
            return await this.findOne({
                where: [
                    {
                        token: credential
                    }
                ]
            });
        } catch (e) {
            return null;
        }
    }

    async deleteChangePasswordToken(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .delete()
                .from(ChangePasswordToken)
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return false;
        }
    }
}