import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { UploadedMulterFileI } from "../SpacesModule/SpacesService";
import { DoSpacesService } from "../SpacesModule/SpacesService/doSpacesService";
import { ContactUsSectionOneRepository } from "./contact-us-section-one.repository";
import { CreateContactUsSectionOneDto } from "./dto/create-contact-us-section-one.dto";
import { ContactUsSectionOneEntity } from "./entities/contact-us-section-one.entity";
import { UpdateContactUsSectionOneDto } from "./dto/update-contact-us-section-one.dto";

@Injectable()
export class ContactUsSectionOneService {
    constructor(
        @InjectRepository(ContactUsSectionOneRepository)
        private readonly repository: ContactUsSectionOneRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async create(
        dto: CreateContactUsSectionOneDto,
        files: {
            icon: UploadedMulterFileI
        }
    ): Promise<ContactUsSectionOneEntity | Error> {
        if (files && files.icon) {
            const icon: any = await this.doSpaceService.uploadFile(files.icon[0], "ContactUsSectionOne/");
            dto.icon = icon;
        }
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
                Constants.CREATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
    }

    async findAll(): Promise<ContactUsSectionOneEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<ContactUsSectionOneEntity[] | Error> {
        const data = await this.repository.findByFilterQuery({ id: id });
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }

    async update(
        id: string,
        dto: UpdateContactUsSectionOneDto,
        files: {
            icon: UploadedMulterFileI
        }): Promise<ContactUsSectionOneEntity | Error> {
        const res: any = await this.repository.findOneEntity(id);
        if (!res) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        if (files && files.icon) {
            const icon: any = await this.doSpaceService.uploadFile(files.icon[0], "ContactUsSectionOne/");
            dto.icon = icon;
        }
        const data = await this.repository.updateEntity(id, dto);
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data updated successfully", "data", data);
    }
}
