import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateDispoDifferenceSectionDto } from "./dto/create-dispo-difference-section.dto";
import { UpdateDispoDifferenceSectionDto } from "./dto/update-dispo-difference-section.dto";
import { DispoDifferenceSectionEntity } from "./entities/dispo-difference-section.entity";

@EntityRepository(DispoDifferenceSectionEntity)
export class DispoDifferenceSectionRepository extends Repository<DispoDifferenceSectionEntity> {
    async createEntity(dto: CreateDispoDifferenceSectionDto): Promise<DispoDifferenceSectionEntity | Error> {
        try {
            const {
                title,
                description,
                image,
                link
            } = dto;
            const entity = this.create({
                title,
                description,
                image,
                link
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<DispoDifferenceSectionEntity | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateDispoDifferenceSectionDto): Promise<DispoDifferenceSectionEntity | Error> {
        try {
            await this.update(id, {
                title: dto.title,
                description: dto.description,
                image: dto.image,
                link: dto.link
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<DispoDifferenceSectionEntity[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
                .createQueryBuilder()
                .update(DispoDifferenceSectionEntity)
                .set({ isDeleted: true })
                .where("id = :id", { id: id })
                .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}