import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { DoSpacesService } from "../../../SpacesModule/SpacesService/doSpacesService";
import { RetailerServiceRepository } from "./retailer-service.repository";
import { RetailerService } from "./entities/retailer-service.entity";
import { UploadedMulterFileI } from "../../../SpacesModule/SpacesService";

@Injectable()
export class RetailerServiceService {
    constructor(
        @InjectRepository(RetailerServiceRepository)
        private readonly repository: RetailerServiceRepository,
        private readonly doSpaceService: DoSpacesService
    ) {
    }

    async update(
        id: string,
        dto: any,
        files: {
            retailerImage?: UploadedMulterFileI
        }
    ): Promise<any | Error> {
        if (files && files.retailerImage) {
            const retailerImage: any = await this.doSpaceService.uploadFile(files.retailerImage[0], "RetailerService/");
            dto.retailerImage = retailerImage;
        }
        await this.repository.update(id, {
            retailerImage: dto.retailerImage,
            retailerMobileNumber: dto.retailerMobileNumber
        });
        const data = await this.repository.findOne({ where: { id: id } });
        if (!data) {
            throw new HttpException(
                Constants.UPDATE_FAILED,
                HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data Updated successfully", "data", data);
    }

    async findAll(): Promise<RetailerService[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<RetailerService[] | Error> {
        const data = await this.repository.findByFilterQuery({ productId: id });
        const reviewCount = Object.keys(data).length != 0 ? Object.keys(data).length : 0;
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        let ratingAvarage = 0;
        if (Object.keys(data).length > 0) {
            let count = 0;
            for (let i = 0; i < reviewCount; i++) {
                count += parseInt(data[i].rating);
            }
            ratingAvarage = Math.round(count / reviewCount);
        }

        const response = {
            data: data,
            reviewCount: reviewCount,
            ratingAvarage: ratingAvarage
        };
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", response);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }

    async updateAlpineIq(id: string, dto: any): Promise<any | Error> {

        const FedexCutoffTime: any = await this.repository.findOneEntity(id);
        if (!FedexCutoffTime) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        const updateData = await this.repository.update(id, {
            alpineIqApiKey: dto.alpineIqApiKey,
            alpineIqUID: dto.alpineIqUID
        });
        const data = await this.repository.findOne(id);
        if (!data) {
            throw new HttpException(
                Constants.NOT_FOUND,
                HttpStatus.NOT_FOUND
            );
        }

        return ResponseUtils.successResponseHandler(
            200,
            "Data Updated Successful!",
            "data",
            data);
    }
}
