import { Body, Controller, Delete, Get, Param, Patch, Post, UseGuards } from "@nestjs/common";
import { Constants } from "src/utils/constants";
import { AdminAuthGuard } from "src/core/guards/admin-auth.guard";
import { HeroSectionSliderService } from "./hero-section-slider.service";
import { CreateHeroSectionSliderDto } from "./dto/create-hero-section-slider.dto";
import { UploadedFiles, UseInterceptors } from "@nestjs/common/decorators";
import { FileFieldsInterceptor } from "@nestjs/platform-express";
import { UploadedMulterFileI } from "../SpacesModule/SpacesService";
import { UpdateHeroSectionSliderDto } from "./dto/update-hero-section-slider.dto";

@Controller({ path: "hero-section-slider", version: Constants.API_VERSION_1 })

export class HeroSectionSliderController {
    constructor(private readonly service: HeroSectionSliderService) {
    }

    @Post()
    // @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: "image", maxCount: 1 }
        ]
    ))
    create(
        @Body() dto: CreateHeroSectionSliderDto,
        @UploadedFiles() files: {
            image: UploadedMulterFileI
        }
    ) {
        return this.service.create(dto, files);
    }

    @Get()
    @UseGuards(AdminAuthGuard)
    findAll() {
        return this.service.findAll();
    }

    @Get(":id")
    @UseGuards(AdminAuthGuard)
    findOne(@Param("id") id: string) {
        return this.service.findOne(id);
    }

    @Patch("/:id")
    @UseGuards(AdminAuthGuard)
    @UseInterceptors(FileFieldsInterceptor(
        [
            { name: "image", maxCount: 1 }
        ]
    ))
    update(
        @Param("id") id: string,
        @Body() dto: UpdateHeroSectionSliderDto,
        @UploadedFiles() files: {
            image: UploadedMulterFileI
        }) {
        return this.service.update(id, dto, files);
    }

    @Delete(":id")
    @UseGuards(AdminAuthGuard)
    async remove(@Param("id") id: string) {
        return await this.service.remove(id);
    }
}
